From: tip-bot for Wang Nan <tipbot@zytor.com>
To: linux-tip-commits@vger.kernel.org
Cc: tglx@linutronix.de, lizefan@huawei.com, hpa@zytor.com,
masami.hiramatsu.pt@hitachi.com, ast@kernel.org,
brendan.d.gregg@gmail.com, mingo@kernel.org, jolsa@kernel.org,
acme@redhat.com, wangnan0@huawei.com,
linux-kernel@vger.kernel.org, peterz@infradead.org,
namhyung@kernel.org
Subject: [tip:perf/core] perf data: Support converting data from bpf_perf_event_output()
Date: Sat, 5 Mar 2016 00:15:06 -0800 [thread overview]
Message-ID: <tip-6122d57e9f7c6cb0f0aa276fbd3a12e3af826ef2@git.kernel.org> (raw)
In-Reply-To: <1456479154-136027-9-git-send-email-wangnan0@huawei.com>
Commit-ID: 6122d57e9f7c6cb0f0aa276fbd3a12e3af826ef2
Gitweb: http://git.kernel.org/tip/6122d57e9f7c6cb0f0aa276fbd3a12e3af826ef2
Author: Wang Nan <wangnan0@huawei.com>
AuthorDate: Fri, 26 Feb 2016 09:31:56 +0000
Committer: Arnaldo Carvalho de Melo <acme@redhat.com>
CommitDate: Thu, 3 Mar 2016 11:10:34 -0300
perf data: Support converting data from bpf_perf_event_output()
bpf_perf_event_output() outputs data through sample->raw_data. This
patch adds support to convert those data into CTF. A python script then
can be used to process output data from BPF programs.
Test result:
# cat ./test_bpf_output_2.c
/************************ BEGIN **************************/
#include <uapi/linux/bpf.h>
struct bpf_map_def {
unsigned int type;
unsigned int key_size;
unsigned int value_size;
unsigned int max_entries;
};
#define SEC(NAME) __attribute__((section(NAME), used))
static u64 (*ktime_get_ns)(void) =
(void *)BPF_FUNC_ktime_get_ns;
static int (*trace_printk)(const char *fmt, int fmt_size, ...) =
(void *)BPF_FUNC_trace_printk;
static int (*get_smp_processor_id)(void) =
(void *)BPF_FUNC_get_smp_processor_id;
static int (*perf_event_output)(void *, struct bpf_map_def *, int, void *, unsigned long) =
(void *)BPF_FUNC_perf_event_output;
struct bpf_map_def SEC("maps") channel = {
.type = BPF_MAP_TYPE_PERF_EVENT_ARRAY,
.key_size = sizeof(int),
.value_size = sizeof(u32),
.max_entries = __NR_CPUS__,
};
static inline int __attribute__((always_inline))
func(void *ctx, int type)
{
struct {
u64 ktime;
int type;
} __attribute__((packed)) output_data;
char error_data[] = "Error: failed to output\n";
int err;
output_data.type = type;
output_data.ktime = ktime_get_ns();
err = perf_event_output(ctx, &channel, get_smp_processor_id(),
&output_data, sizeof(output_data));
if (err)
trace_printk(error_data, sizeof(error_data));
return 0;
}
SEC("func_begin=sys_nanosleep")
int func_begin(void *ctx) {return func(ctx, 1);}
SEC("func_end=sys_nanosleep%return")
int func_end(void *ctx) { return func(ctx, 2);}
char _license[] SEC("license") = "GPL";
int _version SEC("version") = LINUX_VERSION_CODE;
/************************* END ***************************/
# ./perf record -e bpf-output/no-inherit,name=evt/ \
-e ./test_bpf_output_2.c/map:channel.event=evt/ \
usleep 100000
[ perf record: Woken up 1 times to write data ]
[ perf record: Captured and wrote 0.012 MB perf.data (2 samples) ]
# ./perf script
usleep 14942 92503.198504: evt: ffffffff810e0ba1 sys_nanosleep (/lib/modules/4.3.0....
usleep 14942 92503.298562: evt: ffffffff810585e9 kretprobe_trampoline_holder (/lib....
# ./perf data convert --to-ctf ./out.ctf
[ perf data convert: Converted 'perf.data' into CTF data './out.ctf' ]
[ perf data convert: Converted and wrote 0.000 MB (2 samples) ]
# babeltrace ./out.ctf
[01:41:43.198504134] (+?.?????????) evt: { cpu_id = 0 }, { perf_ip = 0xFFFFFFFF810E0BA1, perf_tid = 14942, perf_pid = 14942, perf_id = 1044, raw_len = 3, raw_data = [ [0] = 0x32C0C07B, [1] = 0x5421, [2] = 0x1 ] }
[01:41:43.298562257] (+0.100058123) evt: { cpu_id = 0 }, { perf_ip = 0xFFFFFFFF810585E9, perf_tid = 14942, perf_pid = 14942, perf_id = 1044, raw_len = 3, raw_data = [ [0] = 0x38B77FAA, [1] = 0x5421, [2] = 0x2 ] }
# cat ./test_bpf_output_2.py
from babeltrace import TraceCollection
tc = TraceCollection()
tc.add_trace('./out.ctf', 'ctf')
d = {1:[], 2:[]}
for event in tc.events:
if not event.name.startswith('evt'):
continue
raw_data = event['raw_data']
(time, type) = ((raw_data[0] + (raw_data[1] << 32)), raw_data[2])
d[type].append(time)
print(list(map(lambda i: d[2][i] - d[1][i], range(len(d[1])))));
# python3 ./test_bpf_output_2.py
[100056879]
Committer note:
Make sure you have python3-devel installed, not python-devel, which may
be for python2, which will lead to some "PyInstance_Type" errors. Also
make sure that you use the right libbabeltrace, because it is shipped
in Fedora, for instance, but an older version.
To build libbabeltrace's python binding one also needs to use:
./configure --enable-python-bindings
And then set PYTHONPATH=/usr/local/lib64/python3.4/site-packages/.
Signed-off-by: Wang Nan <wangnan0@huawei.com>
Tested-by: Arnaldo Carvalho de Melo <acme@redhat.com>
Acked-by: Jiri Olsa <jolsa@kernel.org>
Cc: Alexei Starovoitov <ast@kernel.org>
Cc: Brendan Gregg <brendan.d.gregg@gmail.com>
Cc: Li Zefan <lizefan@huawei.com>
Cc: Masami Hiramatsu <masami.hiramatsu.pt@hitachi.com>
Cc: Namhyung Kim <namhyung@kernel.org>
Cc: Peter Zijlstra <peterz@infradead.org>
Cc: Zefan Li <lizefan@huawei.com>
Cc: pi3orama@163.com
Link: http://lkml.kernel.org/r/1456479154-136027-9-git-send-email-wangnan0@huawei.com
Signed-off-by: Arnaldo Carvalho de Melo <acme@redhat.com>
---
tools/perf/util/data-convert-bt.c | 112 +++++++++++++++++++++++++++++++++++++-
1 file changed, 111 insertions(+), 1 deletion(-)
diff --git a/tools/perf/util/data-convert-bt.c b/tools/perf/util/data-convert-bt.c
index 6729f4d..1f608a6 100644
--- a/tools/perf/util/data-convert-bt.c
+++ b/tools/perf/util/data-convert-bt.c
@@ -352,6 +352,84 @@ static int add_tracepoint_values(struct ctf_writer *cw,
return ret;
}
+static int
+add_bpf_output_values(struct bt_ctf_event_class *event_class,
+ struct bt_ctf_event *event,
+ struct perf_sample *sample)
+{
+ struct bt_ctf_field_type *len_type, *seq_type;
+ struct bt_ctf_field *len_field, *seq_field;
+ unsigned int raw_size = sample->raw_size;
+ unsigned int nr_elements = raw_size / sizeof(u32);
+ unsigned int i;
+ int ret;
+
+ if (nr_elements * sizeof(u32) != raw_size)
+ pr_warning("Incorrect raw_size (%u) in bpf output event, skip %lu bytes\n",
+ raw_size, nr_elements * sizeof(u32) - raw_size);
+
+ len_type = bt_ctf_event_class_get_field_by_name(event_class, "raw_len");
+ len_field = bt_ctf_field_create(len_type);
+ if (!len_field) {
+ pr_err("failed to create 'raw_len' for bpf output event\n");
+ ret = -1;
+ goto put_len_type;
+ }
+
+ ret = bt_ctf_field_unsigned_integer_set_value(len_field, nr_elements);
+ if (ret) {
+ pr_err("failed to set field value for raw_len\n");
+ goto put_len_field;
+ }
+ ret = bt_ctf_event_set_payload(event, "raw_len", len_field);
+ if (ret) {
+ pr_err("failed to set payload to raw_len\n");
+ goto put_len_field;
+ }
+
+ seq_type = bt_ctf_event_class_get_field_by_name(event_class, "raw_data");
+ seq_field = bt_ctf_field_create(seq_type);
+ if (!seq_field) {
+ pr_err("failed to create 'raw_data' for bpf output event\n");
+ ret = -1;
+ goto put_seq_type;
+ }
+
+ ret = bt_ctf_field_sequence_set_length(seq_field, len_field);
+ if (ret) {
+ pr_err("failed to set length of 'raw_data'\n");
+ goto put_seq_field;
+ }
+
+ for (i = 0; i < nr_elements; i++) {
+ struct bt_ctf_field *elem_field =
+ bt_ctf_field_sequence_get_field(seq_field, i);
+
+ ret = bt_ctf_field_unsigned_integer_set_value(elem_field,
+ ((u32 *)(sample->raw_data))[i]);
+
+ bt_ctf_field_put(elem_field);
+ if (ret) {
+ pr_err("failed to set raw_data[%d]\n", i);
+ goto put_seq_field;
+ }
+ }
+
+ ret = bt_ctf_event_set_payload(event, "raw_data", seq_field);
+ if (ret)
+ pr_err("failed to set payload for raw_data\n");
+
+put_seq_field:
+ bt_ctf_field_put(seq_field);
+put_seq_type:
+ bt_ctf_field_type_put(seq_type);
+put_len_field:
+ bt_ctf_field_put(len_field);
+put_len_type:
+ bt_ctf_field_type_put(len_type);
+ return ret;
+}
+
static int add_generic_values(struct ctf_writer *cw,
struct bt_ctf_event *event,
struct perf_evsel *evsel,
@@ -597,6 +675,12 @@ static int process_sample_event(struct perf_tool *tool,
return -1;
}
+ if (perf_evsel__is_bpf_output(evsel)) {
+ ret = add_bpf_output_values(event_class, event, sample);
+ if (ret)
+ return -1;
+ }
+
cs = ctf_stream(cw, get_sample_cpu(cw, sample, evsel));
if (cs) {
if (is_flush_needed(cs))
@@ -744,6 +828,25 @@ static int add_tracepoint_types(struct ctf_writer *cw,
return ret;
}
+static int add_bpf_output_types(struct ctf_writer *cw,
+ struct bt_ctf_event_class *class)
+{
+ struct bt_ctf_field_type *len_type = cw->data.u32;
+ struct bt_ctf_field_type *seq_base_type = cw->data.u32_hex;
+ struct bt_ctf_field_type *seq_type;
+ int ret;
+
+ ret = bt_ctf_event_class_add_field(class, len_type, "raw_len");
+ if (ret)
+ return ret;
+
+ seq_type = bt_ctf_field_type_sequence_create(seq_base_type, "raw_len");
+ if (!seq_type)
+ return -1;
+
+ return bt_ctf_event_class_add_field(class, seq_type, "raw_data");
+}
+
static int add_generic_types(struct ctf_writer *cw, struct perf_evsel *evsel,
struct bt_ctf_event_class *event_class)
{
@@ -755,7 +858,8 @@ static int add_generic_types(struct ctf_writer *cw, struct perf_evsel *evsel,
* ctf event header
* PERF_SAMPLE_READ - TODO
* PERF_SAMPLE_CALLCHAIN - TODO
* PERF_SAMPLE_RAW - tracepoint fields are handled separately
* PERF_SAMPLE_BRANCH_STACK - TODO
* PERF_SAMPLE_REGS_USER - TODO
* PERF_SAMPLE_STACK_USER - TODO
@@ -824,6 +928,12 @@ static int add_event(struct ctf_writer *cw, struct perf_evsel *evsel)
goto err;
}
+ if (perf_evsel__is_bpf_output(evsel)) {
+ ret = add_bpf_output_types(cw, event_class);
+ if (ret)
+ goto err;
+ }
+
ret = bt_ctf_stream_class_add_event_class(cw->stream_class, event_class);
if (ret) {
pr("Failed to add event class into stream.\n");
next prev parent reply other threads:[~2016-03-05 8:16 UTC|newest]
Thread overview: 60+ messages / expand[flat|nested] mbox.gz Atom feed top
2016-02-26 9:31 [PATCH 00/46] perf tools: Fix and improvements (bpf and overwrite) Wang Nan
2016-02-26 9:31 ` [PATCH 01/46] perf tools: Record text offset in dso to calculate objdump address Wang Nan
2016-03-24 7:37 ` [tip:perf/urgent] perf symbols: " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 02/46] perf tools: Adjust symbol for shared objects Wang Nan
2016-02-26 9:31 ` [PATCH 03/46] perf config: Bring perf_default_config to the very beginning at main() Wang Nan
2016-02-27 9:44 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 04/46] perf trace: Improve error message when receive non-tracepoint events Wang Nan
2016-02-26 9:31 ` [PATCH 05/46] perf tools: Only set filter for tracepoints events Wang Nan
2016-02-27 9:45 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 06/46] perf trace: Call bpf__apply_obj_config in 'perf trace' Wang Nan
2016-02-27 9:45 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 07/46] perf trace: Print content of bpf-output event Wang Nan
2016-02-27 9:45 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 08/46] perf data: Support converting data from bpf_perf_event_output() Wang Nan
2016-03-05 8:15 ` tip-bot for Wang Nan [this message]
2016-02-26 9:31 ` [PATCH 09/46] perf data: Explicitly set byte order for integer types Wang Nan
2016-03-05 8:15 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:31 ` [PATCH 10/46] perf core: Introduce new ioctl options to pause and resume ring buffer Wang Nan
2016-02-29 15:39 ` Arnaldo Carvalho de Melo
2016-03-03 2:03 ` Wangnan (F)
2016-02-26 9:31 ` [PATCH 11/46] perf core: Set event's default overflow_handler Wang Nan
2016-02-26 9:32 ` [PATCH 12/46] perf core: Prepare writing into ring buffer from end Wang Nan
2016-02-26 9:32 ` [PATCH 13/46] perf core: Add backward attribute to perf event Wang Nan
2016-02-26 9:32 ` [PATCH 14/46] perf core: Reduce perf event output overhead by new overflow handler Wang Nan
2016-02-26 9:32 ` [PATCH 15/46] perf tools: Only validate is_pos for tracking evsels Wang Nan
2016-02-26 9:32 ` [PATCH 16/46] perf tools: Print write_backward value in perf_event_attr__fprintf Wang Nan
2016-02-26 9:32 ` [PATCH 17/46] perf tools: Make ordered_events reusable Wang Nan
2016-02-26 9:32 ` [PATCH 18/46] perf record: Use WARN_ONCE to replace 'if' condition Wang Nan
2016-03-05 8:15 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:32 ` [PATCH 19/46] perf record: Extract synthesize code to record__synthesize() Wang Nan
2016-03-05 8:16 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:32 ` [PATCH 20/46] perf tools: Add perf_data_file__switch() helper Wang Nan
2016-02-26 9:32 ` [PATCH 21/46] perf record: Turns auxtrace_snapshot_enable into 3 states Wang Nan
2016-02-26 9:32 ` [PATCH 22/46] perf record: Introduce record__finish_output() to finish a perf.data Wang Nan
2016-03-05 8:16 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:32 ` [PATCH 23/46] perf record: Add '--timestamp-filename' option to append timestamp to output filename Wang Nan
2016-02-26 9:32 ` [PATCH 24/46] perf record: Split output into multiple files via '--switch-output' Wang Nan
2016-02-26 9:32 ` [PATCH 25/46] perf record: Force enable --timestamp-filename when --switch-output is provided Wang Nan
2016-02-26 9:32 ` [PATCH 26/46] perf record: Disable buildid cache options by default in switch output mode Wang Nan
2016-02-26 9:32 ` [PATCH 27/46] perf record: Re-synthesize tracking events after output switching Wang Nan
2016-02-26 9:32 ` [PATCH 28/46] perf record: Generate tracking events for process forked by perf Wang Nan
2016-02-26 9:32 ` [PATCH 29/46] perf record: Ensure return non-zero rc when mmap fail Wang Nan
2016-03-05 8:17 ` [tip:perf/core] " tip-bot for Wang Nan
2016-02-26 9:32 ` [PATCH 30/46] perf record: Prevent reading invalid data in record__mmap_read Wang Nan
2016-02-26 9:32 ` [PATCH 31/46] perf tools: Add evlist channel helpers Wang Nan
2016-02-26 9:32 ` [PATCH 32/46] perf tools: Automatically add new channel according to evlist Wang Nan
2016-02-26 9:32 ` [PATCH 33/46] perf tools: Operate multiple channels Wang Nan
2016-02-26 9:32 ` [PATCH 34/46] perf tools: Squash overwrite setting into channel Wang Nan
2016-02-26 9:32 ` [PATCH 35/46] perf record: Don't read from and poll overwrite channel Wang Nan
2016-02-26 9:32 ` [PATCH 36/46] perf record: Don't poll on " Wang Nan
2016-02-26 9:32 ` [PATCH 37/46] perf tools: Detect avalibility of write_backward Wang Nan
2016-02-26 9:32 ` [PATCH 38/46] perf tools: Enable overwrite settings Wang Nan
2016-02-26 9:32 ` [PATCH 39/46] perf tools: Set write_backward attribut bit for overwrite events Wang Nan
2016-02-26 9:32 ` [PATCH 40/46] perf tools: Record fd into perf_mmap Wang Nan
2016-02-26 9:32 ` [PATCH 41/46] perf tools: Add API to pause a channel Wang Nan
2016-02-26 9:32 ` [PATCH 42/46] perf record: Toggle overwrite ring buffer for reading Wang Nan
2016-02-26 9:32 ` [PATCH 43/46] perf record: Rename variable to make code clear Wang Nan
2016-02-26 9:32 ` [PATCH 44/46] perf record: Read from backward ring buffer Wang Nan
2016-02-26 9:32 ` [PATCH 45/46] perf record: Allow generate tracking events at the end of output Wang Nan
2016-02-26 9:32 ` [PATCH 46/46] perf tools: Don't warn about out of order event if write_backward is used Wang Nan
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=tip-6122d57e9f7c6cb0f0aa276fbd3a12e3af826ef2@git.kernel.org \
--to=tipbot@zytor.com \
--cc=acme@redhat.com \
--cc=ast@kernel.org \
--cc=brendan.d.gregg@gmail.com \
--cc=hpa@zytor.com \
--cc=jolsa@kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-tip-commits@vger.kernel.org \
--cc=lizefan@huawei.com \
--cc=masami.hiramatsu.pt@hitachi.com \
--cc=mingo@kernel.org \
--cc=namhyung@kernel.org \
--cc=peterz@infradead.org \
--cc=tglx@linutronix.de \
--cc=wangnan0@huawei.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -755,7 +858,8 @@ static int add_generic_types(struct ctf_writer *cw, struct perf_evsel *evsel,
* PERF_SAMPLE_READ - TODO
* PERF_SAMPLE_CALLCHAIN - TODO
- * PERF_SAMPLE_RAW - tracepoint fields are handled separately
+ * PERF_SAMPLE_RAW - tracepoint fields and BPF output
+ * are handled separately
* PERF_SAMPLE_BRANCH_STACK - TODO
* PERF_SAMPLE_REGS_USER - TODO
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox